#include <Servo.h>
#include <SoftwareSerial.h>

//Anpassen:
const int minWert = 180;
const int maxWert = 220;
const int k = 20;
const unsigned int test = 0;

//Pins:
int M1In1 = 5;
int M1In2 = 6;
int M2In1 = 11;
int M2In2 = 12;

int M1EncA = 2;
int M1EncB = 8;
int M2EncA = 3;
int M2EncB = 7;

//Motor 1
unsigned long encoderPos1 = 0;
unsigned long messzeit1Alt = 0;
float messgeschwindigkeit1;
int hall1BAlt;
boolean ersteMessung1;
float motorAusgang1 = 200;
boolean fahrtRichtung1 = false;

//Motor 2
unsigned long encoderPos2 = 0;
unsigned long messzeit2Alt = 0;
float messgeschwindigkeit2;
int hall2BAlt;
boolean ersteMessung2;
float motorAusgang2 = 200;
boolean fahrtRichtung2 = true;

int a = 0;
float umdrehungen = 0;
boolean fahren = false;

Servo myservo;
SoftwareSerial mySerial(A0,A1);

void setup()
{
  pinMode(M1In1, OUTPUT); //In1 Motor 1
  pinMode(M1In2, OUTPUT); //In2 Motor 1
  pinMode(M2In1, OUTPUT); //In1 Motor 2
  pinMode(M2In2, OUTPUT); //In2 Motor 2

  pinMode(M1EncA, INPUT); //Motor 1 Encoder A
  pinMode(M1EncB, INPUT); //Motor 1 Encoder B
  pinMode(M2EncA, INPUT); //Motor 2 Encoder A
  pinMode(M2EncB, INPUT); //Motor 2 Encoder B

  attachInterrupt(digitalPinToInterrupt(M1EncA), Hall1Change, CHANGE); //Interrupt durch den Encoder an Motor 1
  attachInterrupt(digitalPinToInterrupt(M2EncA), Hall2Change, CHANGE); //Interrupt durch den Encoder an Motor 2

  myservo.attach(9);

  Serial.begin(19200);
  mySerial.begin(19200);
  Serial.println("Start");

  messgeschwindigkeit1 = 0;
  messgeschwindigkeit2 = 0;

}

void loop()
{
  if (test == 1)
  {
      if (a >= 10000)
      {
      Serial.print(messgeschwindigkeit1);
      Serial.print('\t');
      Serial.print(encoderPos1);
      Serial.print('\t');
      Serial.print(encoderPos1 / 2370.72);
      Serial.print('\t');
      Serial.print(motorAusgang1);
      Serial.print('\t');
      Serial.print(messgeschwindigkeit2);
      Serial.print('\t');
      Serial.print(encoderPos2);
      Serial.print('\t');
      Serial.print(encoderPos2 / 2370.72);
      Serial.print('\t');
      Serial.print(motorAusgang2);
      Serial.print('\n');

      a = 0;
      }
      a++;
  }

  if ((encoderPos1 > umdrehungen * 2370.72) || (encoderPos2 > umdrehungen * 2370.72))
  {
    encoderPos1 = 0;
    encoderPos2 = 0;
    fahren = false;
    mySerial.write('f');
  }
  Eingabe();
  motorSteuerung();
}

void Eingabe()
{
  if (mySerial.available())    // Wenn Daten empfangen wurden und zum Lesen bereitstehen
  {
    int val = mySerial.read();  // lese das erste Zeichen in der Warteschlange und speichere es in der Variable 'val' zwischen
    switch (val)
    {
     case 'R':     // schwamm hoch
        for (int i = 20; i < 160; i++)
        {
          myservo.write(i);
          delay(20);
        }
        break;
      case 'H': //schwamm runter
        for (int i = 160; i > 20; i--)
        {
          myservo.write(i);
          delay(20);
        }
        break;
      case 'v': //vorwaerts fahren
        fahren = true;
        fahrtRichtung1 = true;
        fahrtRichtung2 = true;
        motorAusgang1 = 200;
        motorAusgang2 = 200;
        umdrehungen = mySerial.parseFloat();
        break;
      case 'b': //ruwckwaerts fahren
        fahren = true;
        fahrtRichtung1 = false;
        fahrtRichtung2 = false;
        motorAusgang1 = 200;
        motorAusgang2 = 200;
        umdrehungen = mySerial.parseFloat();
        break;
      case 'r': //rechts drehen
        fahren = true;
        fahrtRichtung1 = true;
        fahrtRichtung2 = false;
        motorAusgang1 = 200;
        motorAusgang2 = 200;
        umdrehungen = mySerial.parseFloat();
        break;
      case 'l': //links drehen
        fahren = true;
        fahrtRichtung1 = false;
        fahrtRichtung2 = true;
        motorAusgang1 = 200;
        motorAusgang2 = 200;
        umdrehungen = mySerial.parseFloat();
        break;
      case 's':
        fahren = false;
        umdrehungen = 0;
        break;
      default: 
        break;
    }
    mySerial.write(val);
  }
}
void Hall1Change()
{
  boolean illegal = false;

  //Encoderwerte einlesen
  int hall1A = digitalRead(M1EncA);
  int hall1B = digitalRead(M1EncB);

  if (ersteMessung1 == true)
  {
    illegal = true;
    ersteMessung1 = false;
  }
  if (hall1BAlt == hall1B)
  {
    illegal = true;
  }

  unsigned long currTime1 = micros();
  if (illegal == false)
  {
    messgeschwindigkeit1 = 1000000 / float((currTime1 - messzeit1Alt) * 24 * 98.78);
    if (hall1A == 1)
    {
      if (hall1B == 1)
      {
        messgeschwindigkeit1 *= -1;
      }
    }
    else
    {
      if (hall1B == 0)
      {
        messgeschwindigkeit1 *= -1;
      }
    }
    encoderPos1 += 1;
  }

  //Vergleichsvariablen neu beschreiben
  hall1BAlt = hall1B;
  messzeit1Alt = currTime1;
}

void Hall2Change()
{
  boolean illegal = false;

  //Encoderwerte einlesen
  int hall2A = digitalRead(M2EncA);
  int hall2B = digitalRead(M2EncB);

  if (ersteMessung2 == true)
  {
    illegal = true;
    ersteMessung2 = false;
  }
  if (hall2BAlt == hall2B)
  {
    illegal = true;
  }

  unsigned long currTime2 = micros();
  if (illegal == false)
  {
    messgeschwindigkeit2 = 1000000 / float((currTime2 - messzeit2Alt) * 24 * 98.78);
    if (hall2A == 0)
    {
      if (hall2B == 1)
      {
        messgeschwindigkeit2 *= -1;
      }
    }
    else
    {
      if (hall2B == 0)
      {
        messgeschwindigkeit2 *= -1;
      }
    }
    encoderPos2 += 1;
  }

  //Vergleichsvariablen neu beschreiben
  hall2BAlt = hall2B;
  messzeit2Alt = currTime2;
}


int regelung()
{
  float difGeschwindigkeit = abs(abs(messgeschwindigkeit1) - abs(messgeschwindigkeit2));
  float x =  k * difGeschwindigkeit;

  if (abs(messgeschwindigkeit1) > abs(messgeschwindigkeit2))
    x *= -1;
  else if (abs(messgeschwindigkeit1) == abs(messgeschwindigkeit2))
    return 1;

  motorAusgang1 += x;

  int differenz = motorAusgang1 - motorAusgang2;

  if (differenz < 0) //motorAusgang1 < motorAusgang2
  {
    if (motorAusgang1 < minWert)
    {
      motorAusgang1 = minWert;
      motorAusgang2 = minWert + abs(differenz);
    }
    else if (motorAusgang2 > maxWert)
    {
      motorAusgang2 = maxWert;
      motorAusgang1 = maxWert - abs(differenz);
    }
  }
  else if (differenz > 0) //motorAusgang1 > motorAusgang2
  {
    if (motorAusgang2 < minWert)
    {
      motorAusgang2 = minWert;
      motorAusgang1 = minWert + abs(differenz);
    }
    else if (motorAusgang1 > maxWert)
    {
      motorAusgang1 = maxWert;
      motorAusgang2 = maxWert - abs(differenz);
    }
  }
  constrain(motorAusgang1, 0, 255);
  constrain(motorAusgang2, 0, 255);
  if (differenz < 255)
  {
    if (motorAusgang1 == 255)
      motorAusgang2 -= abs(differenz);
    else if (motorAusgang1 == 0)
      motorAusgang2 += abs(differenz);
    if (motorAusgang2 == 255)
      motorAusgang1 -= abs(differenz);
    else if (motorAusgang2 == 0)
      motorAusgang1 += abs(differenz);
  }
  return 1;
}

void motorSteuerung()
{
  if (fahren == true)
  {
    regelung();
    if (fahrtRichtung1 == true)
    {
      analogWrite(M1In1, int(motorAusgang1));
      digitalWrite(M1In2, LOW);
    }
    else
    {
      digitalWrite(M1In1, LOW);
      analogWrite(M1In2, int(motorAusgang1));
    }

    if (fahrtRichtung2 == true)
    {
      analogWrite(M2In1, int(motorAusgang2));
      digitalWrite(M2In2, LOW);
    }
    else
    {
      digitalWrite(M2In1, LOW);
      analogWrite(M2In2, int(motorAusgang2));
    }
  }
  else
  {
    digitalWrite(M1In1, LOW);
    digitalWrite(M1In2, LOW);
    digitalWrite(M2In1, LOW);
    digitalWrite(M2In2, LOW);
  }
}

